@@ -5,6 +5,7 @@
  */
 
 #include <assert.h>
+#include <cbor.h>
 #include <fido.h>
 #include <string.h>
 
@@ -231,6 +232,16 @@ const unsigned char id[64] = {
 	0x34, 0xe3, 0x83, 0xe7, 0xd1, 0xbd, 0x9f, 0x25,
 };
 
+/*
+ * Security Key By Yubico
+ * 5.1.X
+ * f8a011f3-8c0a-4d15-8006-17111f9edc7d
+*/
+const unsigned char aaguid[16] = {
+	0xf8, 0xa0, 0x11, 0xf3, 0x8c, 0x0a, 0x4d, 0x15,
+	0x80, 0x06, 0x17, 0x11, 0x1f, 0x9e, 0xdc, 0x7d,
+};
+
 const char rp_id[] = "localhost";
 const char rp_name[] = "sweet home localhost";
 
@@ -317,12 +328,15 @@ empty_cred(void)
 	c = alloc_cred();
 	assert(fido_cred_authdata_len(c) == 0);
 	assert(fido_cred_authdata_ptr(c) == NULL);
+	assert(fido_cred_authdata_raw_len(c) == 0);
+	assert(fido_cred_authdata_raw_ptr(c) == NULL);
 	assert(fido_cred_clientdata_hash_len(c) == 0);
 	assert(fido_cred_clientdata_hash_ptr(c) == NULL);
 	assert(fido_cred_flags(c) == 0);
 	assert(fido_cred_fmt(c) == NULL);
 	assert(fido_cred_id_len(c) == 0);
 	assert(fido_cred_id_ptr(c) == NULL);
+	assert(fido_cred_prot(c) == 0);
 	assert(fido_cred_pubkey_len(c) == 0);
 	assert(fido_cred_pubkey_ptr(c) == NULL);
 	assert(fido_cred_rp_id(c) == NULL);
@@ -374,10 +388,13 @@ valid_cred(void)
 	assert(fido_cred_set_sig(c, sig, sizeof(sig)) == FIDO_OK);
 	assert(fido_cred_set_fmt(c, "packed") == FIDO_OK);
 	assert(fido_cred_verify(c) == FIDO_OK);
+	assert(fido_cred_prot(c) == 0);
 	assert(fido_cred_pubkey_len(c) == sizeof(pubkey));
 	assert(memcmp(fido_cred_pubkey_ptr(c), pubkey, sizeof(pubkey)) == 0);
 	assert(fido_cred_id_len(c) == sizeof(id));
 	assert(memcmp(fido_cred_id_ptr(c), id, sizeof(id)) == 0);
+	assert(fido_cred_aaguid_len(c) == sizeof(aaguid));
+	assert(memcmp(fido_cred_aaguid_ptr(c), aaguid, sizeof(aaguid)) == 0);
 	free_cred(c);
 }
 
@@ -400,6 +417,8 @@ no_cdh(void)
 	assert(memcmp(fido_cred_pubkey_ptr(c), pubkey, sizeof(pubkey)) == 0);
 	assert(fido_cred_id_len(c) == sizeof(id));
 	assert(memcmp(fido_cred_id_ptr(c), id, sizeof(id)) == 0);
+	assert(fido_cred_aaguid_len(c) == sizeof(aaguid));
+	assert(memcmp(fido_cred_aaguid_ptr(c), aaguid, sizeof(aaguid)) == 0);
 	free_cred(c);
 }
 
@@ -422,6 +441,8 @@ no_rp_id(void)
 	assert(memcmp(fido_cred_pubkey_ptr(c), pubkey, sizeof(pubkey)) == 0);
 	assert(fido_cred_id_len(c) == sizeof(id));
 	assert(memcmp(fido_cred_id_ptr(c), id, sizeof(id)) == 0);
+	assert(fido_cred_aaguid_len(c) == sizeof(aaguid));
+	assert(memcmp(fido_cred_aaguid_ptr(c), aaguid, sizeof(aaguid)) == 0);
 	free_cred(c);
 }
 
@@ -445,6 +466,8 @@ no_rp_name(void)
 	assert(memcmp(fido_cred_pubkey_ptr(c), pubkey, sizeof(pubkey)) == 0);
 	assert(fido_cred_id_len(c) == sizeof(id));
 	assert(memcmp(fido_cred_id_ptr(c), id, sizeof(id)) == 0);
+	assert(fido_cred_aaguid_len(c) == sizeof(aaguid));
+	assert(memcmp(fido_cred_aaguid_ptr(c), aaguid, sizeof(aaguid)) == 0);
 	free_cred(c);
 }
 
@@ -452,6 +475,10 @@ static void
 no_authdata(void)
 {
 	fido_cred_t *c;
+	unsigned char *unset;
+
+	unset = calloc(1, sizeof(aaguid));
+	assert(unset != NULL);
 
 	c = alloc_cred();
 	assert(fido_cred_set_type(c, COSE_ES256) == FIDO_OK);
@@ -467,7 +494,10 @@ no_authdata(void)
 	assert(fido_cred_pubkey_ptr(c) == NULL);
 	assert(fido_cred_id_len(c) == 0);
 	assert(fido_cred_id_ptr(c) == NULL);
+	assert(fido_cred_aaguid_len(c) == sizeof(aaguid));
+	assert(memcmp(fido_cred_aaguid_ptr(c), unset, sizeof(aaguid)) == 0);
 	free_cred(c);
+	free(unset);
 }
 
 static void
@@ -489,6 +519,8 @@ no_x509(void)
 	assert(memcmp(fido_cred_pubkey_ptr(c), pubkey, sizeof(pubkey)) == 0);
 	assert(fido_cred_id_len(c) == sizeof(id));
 	assert(memcmp(fido_cred_id_ptr(c), id, sizeof(id)) == 0);
+	assert(fido_cred_aaguid_len(c) == sizeof(aaguid));
+	assert(memcmp(fido_cred_aaguid_ptr(c), aaguid, sizeof(aaguid)) == 0);
 	free_cred(c);
 }
 
@@ -511,6 +543,8 @@ no_sig(void)
 	assert(memcmp(fido_cred_pubkey_ptr(c), pubkey, sizeof(pubkey)) == 0);
 	assert(fido_cred_id_len(c) == sizeof(id));
 	assert(memcmp(fido_cred_id_ptr(c), id, sizeof(id)) == 0);
+	assert(fido_cred_aaguid_len(c) == sizeof(aaguid));
+	assert(memcmp(fido_cred_aaguid_ptr(c), aaguid, sizeof(aaguid)) == 0);
 	free_cred(c);
 }
 
@@ -533,6 +567,8 @@ no_fmt(void)
 	assert(memcmp(fido_cred_pubkey_ptr(c), pubkey, sizeof(pubkey)) == 0);
 	assert(fido_cred_id_len(c) == sizeof(id));
 	assert(memcmp(fido_cred_id_ptr(c), id, sizeof(id)) == 0);
+	assert(fido_cred_aaguid_len(c) == sizeof(aaguid));
+	assert(memcmp(fido_cred_aaguid_ptr(c), aaguid, sizeof(aaguid)) == 0);
 	free_cred(c);
 }
 
@@ -556,6 +592,8 @@ wrong_options(void)
 	assert(memcmp(fido_cred_pubkey_ptr(c), pubkey, sizeof(pubkey)) == 0);
 	assert(fido_cred_id_len(c) == sizeof(id));
 	assert(memcmp(fido_cred_id_ptr(c), id, sizeof(id)) == 0);
+	assert(fido_cred_aaguid_len(c) == sizeof(aaguid));
+	assert(memcmp(fido_cred_aaguid_ptr(c), aaguid, sizeof(aaguid)) == 0);
 	free_cred(c);
 }
 
@@ -585,6 +623,8 @@ junk_cdh(void)
 	assert(memcmp(fido_cred_pubkey_ptr(c), pubkey, sizeof(pubkey)) == 0);
 	assert(fido_cred_id_len(c) == sizeof(id));
 	assert(memcmp(fido_cred_id_ptr(c), id, sizeof(id)) == 0);
+	assert(fido_cred_aaguid_len(c) == sizeof(aaguid));
+	assert(memcmp(fido_cred_aaguid_ptr(c), aaguid, sizeof(aaguid)) == 0);
 	free_cred(c);
 	free(junk);
 }
@@ -609,6 +649,8 @@ junk_rp_id(void)
 	assert(memcmp(fido_cred_pubkey_ptr(c), pubkey, sizeof(pubkey)) == 0);
 	assert(fido_cred_id_len(c) == sizeof(id));
 	assert(memcmp(fido_cred_id_ptr(c), id, sizeof(id)) == 0);
+	assert(fido_cred_aaguid_len(c) == sizeof(aaguid));
+	assert(memcmp(fido_cred_aaguid_ptr(c), aaguid, sizeof(aaguid)) == 0);
 	free_cred(c);
 }
 
@@ -632,6 +674,8 @@ junk_rp_name(void)
 	assert(memcmp(fido_cred_pubkey_ptr(c), pubkey, sizeof(pubkey)) == 0);
 	assert(fido_cred_id_len(c) == sizeof(id));
 	assert(memcmp(fido_cred_id_ptr(c), id, sizeof(id)) == 0);
+	assert(fido_cred_aaguid_len(c) == sizeof(aaguid));
+	assert(memcmp(fido_cred_aaguid_ptr(c), aaguid, sizeof(aaguid)) == 0);
 	free_cred(c);
 }
 
@@ -640,17 +684,23 @@ junk_authdata(void)
 {
 	fido_cred_t *c;
 	unsigned char *junk;
+	unsigned char *unset;
 
 	junk = malloc(sizeof(authdata));
 	assert(junk != NULL);
 	memcpy(junk, authdata, sizeof(authdata));
 	junk[0] = ~junk[0];
 
+	unset = calloc(1, sizeof(aaguid));
+	assert(unset != NULL);
+
 	c = alloc_cred();
 	assert(fido_cred_set_authdata(c, junk,
 	    sizeof(authdata)) == FIDO_ERR_INVALID_ARGUMENT);
 	assert(fido_cred_authdata_len(c) == 0);
 	assert(fido_cred_authdata_ptr(c) == NULL);
+	assert(fido_cred_authdata_raw_len(c) == 0);
+	assert(fido_cred_authdata_raw_ptr(c) == NULL);
 	assert(fido_cred_flags(c) == 0);
 	assert(fido_cred_fmt(c) == NULL);
 	assert(fido_cred_id_len(c) == 0);
@@ -663,9 +713,12 @@ junk_authdata(void)
 	assert(fido_cred_sig_ptr(c) == NULL);
 	assert(fido_cred_x5c_len(c) == 0);
 	assert(fido_cred_x5c_ptr(c) == NULL);
+	assert(fido_cred_aaguid_len(c) == sizeof(aaguid));
+	assert(memcmp(fido_cred_aaguid_ptr(c), unset, sizeof(aaguid)) == 0);
 	assert(fido_cred_verify(c) == FIDO_ERR_INVALID_ARGUMENT);
 	free_cred(c);
 	free(junk);
+	free(unset);
 }
 
 static void
@@ -694,6 +747,8 @@ junk_sig(void)
 	assert(memcmp(fido_cred_pubkey_ptr(c), pubkey, sizeof(pubkey)) == 0);
 	assert(fido_cred_id_len(c) == sizeof(id));
 	assert(memcmp(fido_cred_id_ptr(c), id, sizeof(id)) == 0);
+	assert(fido_cred_aaguid_len(c) == sizeof(aaguid));
+	assert(memcmp(fido_cred_aaguid_ptr(c), aaguid, sizeof(aaguid)) == 0);
 	free_cred(c);
 	free(junk);
 }
@@ -724,6 +779,8 @@ junk_x509(void)
 	assert(memcmp(fido_cred_pubkey_ptr(c), pubkey, sizeof(pubkey)) == 0);
 	assert(fido_cred_id_len(c) == sizeof(id));
 	assert(memcmp(fido_cred_id_ptr(c), id, sizeof(id)) == 0);
+	assert(fido_cred_aaguid_len(c) == sizeof(aaguid));
+	assert(memcmp(fido_cred_aaguid_ptr(c), aaguid, sizeof(aaguid)) == 0);
 	free_cred(c);
 	free(junk);
 }
@@ -733,6 +790,10 @@ static void
 invalid_type(void)
 {
 	fido_cred_t *c;
+	unsigned char *unset;
+
+	unset = calloc(1, sizeof(aaguid));
+	assert(unset != NULL);
 
 	c = alloc_cred();
 	assert(fido_cred_set_type(c, COSE_RS256) == FIDO_OK);
@@ -749,7 +810,10 @@ invalid_type(void)
 	assert(fido_cred_pubkey_ptr(c) == NULL);
 	assert(fido_cred_id_len(c) == 0);
 	assert(fido_cred_id_ptr(c) == NULL);
+	assert(fido_cred_aaguid_len(c) == sizeof(aaguid));
+	assert(memcmp(fido_cred_aaguid_ptr(c), unset, sizeof(aaguid)) == 0);
 	free_cred(c);
+	free(unset);
 }
 
 /* cbor_serialize_alloc misuse */
@@ -789,6 +853,64 @@ unsorted_keys(void)
 	free_cred(c);
 }
 
+static void
+wrong_credprot(void)
+{
+	fido_cred_t *c;
+
+	c = alloc_cred();
+	assert(fido_cred_set_type(c, COSE_ES256) == FIDO_OK);
+	assert(fido_cred_set_clientdata_hash(c, cdh, sizeof(cdh)) == FIDO_OK);
+	assert(fido_cred_set_rp(c, rp_id, rp_name) == FIDO_OK);
+	assert(fido_cred_set_x509(c, x509, sizeof(x509)) == FIDO_OK);
+	assert(fido_cred_set_sig(c, sig, sizeof(sig)) == FIDO_OK);
+	assert(fido_cred_set_fmt(c, "packed") == FIDO_OK);
+	assert(fido_cred_set_prot(c, FIDO_CRED_PROT_UV_OPTIONAL_WITH_ID) == FIDO_OK);
+	assert(fido_cred_set_authdata(c, authdata, sizeof(authdata)) == FIDO_OK);
+	assert(fido_cred_verify(c) == FIDO_ERR_INVALID_PARAM);
+	free_cred(c);
+}
+
+static void
+raw_authdata(void)
+{
+	fido_cred_t *c;
+	cbor_item_t *item;
+	struct cbor_load_result cbor_result;
+	const unsigned char *ptr;
+	unsigned char *cbor;
+	size_t len;
+	size_t cbor_len;
+	size_t alloclen;
+
+	c = alloc_cred();
+	assert(fido_cred_set_type(c, COSE_ES256) == FIDO_OK);
+	assert(fido_cred_set_authdata(c, authdata, sizeof(authdata)) == FIDO_OK);
+	assert((ptr = fido_cred_authdata_ptr(c)) != NULL);
+	assert((len = fido_cred_authdata_len(c)) != 0);
+	assert((item = cbor_load(ptr, len, &cbor_result)) != NULL);
+	assert(cbor_result.read == len);
+	assert(cbor_isa_bytestring(item));
+	assert((ptr = fido_cred_authdata_raw_ptr(c)) != NULL);
+	assert((len = fido_cred_authdata_raw_len(c)) != 0);
+	assert(cbor_bytestring_length(item) == len);
+	assert(memcmp(ptr, cbor_bytestring_handle(item), len) == 0);
+	assert((len = fido_cred_authdata_len(c)) != 0);
+	assert((cbor_len = cbor_serialize_alloc(item, &cbor, &alloclen)) == len);
+	assert((ptr = cbor_bytestring_handle(item)) != NULL);
+	assert((len = cbor_bytestring_length(item)) != 0);
+	assert(fido_cred_set_authdata_raw(c, ptr, len) == FIDO_OK);
+	assert((ptr = fido_cred_authdata_ptr(c)) != NULL);
+	assert((len = fido_cred_authdata_len(c)) != 0);
+	assert(len == cbor_len);
+	assert(memcmp(cbor, ptr, len) == 0);
+	assert(cbor_len == sizeof(authdata));
+	assert(memcmp(cbor, authdata, cbor_len) == 0);
+	cbor_decref(&item);
+	free(cbor);
+	free_cred(c);
+}
+
 int
 main(void)
 {
@@ -814,6 +936,8 @@ main(void)
 	bad_cbor_serialize();
 	duplicate_keys();
 	unsorted_keys();
+	wrong_credprot();
+	raw_authdata();
 
 	exit(0);
 }
