@@ -313,6 +313,35 @@ fail:
 	return (ok);
 }
 
+static int
+cbor_add_uint8(cbor_item_t *item, const char *key, uint8_t value)
+{
+	struct cbor_pair pair;
+	int ok = -1;
+
+	memset(&pair, 0, sizeof(pair));
+
+	if ((pair.key = cbor_build_string(key)) == NULL ||
+	    (pair.value = cbor_build_uint8(value)) == NULL) {
+		fido_log_debug("%s: cbor_build", __func__);
+		goto fail;
+	}
+
+	if (!cbor_map_add(item, pair)) {
+		fido_log_debug("%s: cbor_map_add", __func__);
+		goto fail;
+	}
+
+	ok = 0;
+fail:
+	if (pair.key)
+		cbor_decref(&pair.key);
+	if (pair.value)
+		cbor_decref(&pair.value);
+
+	return (ok);
+}
+
 static int
 cbor_add_arg(cbor_item_t *item, uint8_t n, cbor_item_t *arg)
 {
@@ -357,7 +386,7 @@ cbor_flatten_vector(cbor_item_t *argv[], size_t argc)
 		return (NULL);
 
 	for (i = 0; i < argc; i++)
-		if (cbor_add_arg(map, i + 1, argv[i]) < 0)
+		if (cbor_add_arg(map, (uint8_t)(i + 1), argv[i]) < 0)
 			break;
 
 	if (i != argc) {
@@ -535,19 +564,31 @@ fail:
 }
 
 cbor_item_t *
-cbor_encode_extensions(int ext)
+cbor_encode_extensions(const fido_cred_ext_t *ext)
 {
 	cbor_item_t *item = NULL;
+	size_t size = 0;
 
-	if (ext == 0 || ext != FIDO_EXT_HMAC_SECRET)
+	if (ext->mask & FIDO_EXT_HMAC_SECRET)
+		size++;
+	if (ext->mask & FIDO_EXT_CRED_PROTECT)
+		size++;
+	if (size == 0 || (item = cbor_new_definite_map(size)) == NULL)
 		return (NULL);
 
-	if ((item = cbor_new_definite_map(1)) == NULL)
-		return (NULL);
-
-	if (cbor_add_bool(item, "hmac-secret", FIDO_OPT_TRUE) < 0) {
-		cbor_decref(&item);
-		return (NULL);
+	if (ext->mask & FIDO_EXT_CRED_PROTECT) {
+		if (ext->prot < 0 || ext->prot > UINT8_MAX ||
+		    cbor_add_uint8(item, "credProtect",
+		    (uint8_t)ext->prot) < 0) {
+			cbor_decref(&item);
+			return (NULL);
+		}
+	}
+	if (ext->mask & FIDO_EXT_HMAC_SECRET) {
+		if (cbor_add_bool(item, "hmac-secret", FIDO_OPT_TRUE) < 0) {
+			cbor_decref(&item);
+			return (NULL);
+		}
 	}
 
 	return (item);
@@ -595,7 +636,7 @@ cbor_encode_pin_auth(const fido_blob_t *hmac_key, const fido_blob_t *data)
 	unsigned int	 dgst_len;
 
 	if ((md = EVP_sha256()) == NULL || HMAC(md, hmac_key->ptr,
-	    (int)hmac_key->len, data->ptr, (int)data->len, dgst,
+	    (int)hmac_key->len, data->ptr, data->len, dgst,
 	    &dgst_len) == NULL || dgst_len != SHA256_DIGEST_LENGTH)
 		return (NULL);
 
@@ -657,7 +698,6 @@ cbor_encode_change_pin_auth(const fido_blob_t *key, const fido_blob_t *new_pin,
 	fido_blob_t	*npe = NULL; /* new pin, encrypted */
 	fido_blob_t	*ph = NULL;  /* pin hash */
 	fido_blob_t	*phe = NULL; /* pin hash, encrypted */
-	int		 ok = -1;
 
 	if ((npe = fido_blob_new()) == NULL ||
 	    (ph = fido_blob_new()) == NULL ||
@@ -686,8 +726,8 @@ cbor_encode_change_pin_auth(const fido_blob_t *key, const fido_blob_t *new_pin,
 
 	if ((md = EVP_sha256()) == NULL ||
 	    HMAC_Init_ex(&ctx, key->ptr, (int)key->len, md, NULL) == 0 ||
-	    HMAC_Update(&ctx, npe->ptr, (int)npe->len) == 0 ||
-	    HMAC_Update(&ctx, phe->ptr, (int)phe->len) == 0 ||
+	    HMAC_Update(&ctx, npe->ptr, npe->len) == 0 ||
+	    HMAC_Update(&ctx, phe->ptr, phe->len) == 0 ||
 	    HMAC_Final(&ctx, dgst, &dgst_len) == 0 || dgst_len != 32) {
 		fido_log_debug("%s: HMAC", __func__);
 		goto fail;
@@ -696,8 +736,8 @@ cbor_encode_change_pin_auth(const fido_blob_t *key, const fido_blob_t *new_pin,
 	if ((ctx = HMAC_CTX_new()) == NULL ||
 	    (md = EVP_sha256())  == NULL ||
 	    HMAC_Init_ex(ctx, key->ptr, (int)key->len, md, NULL) == 0 ||
-	    HMAC_Update(ctx, npe->ptr, (int)npe->len) == 0 ||
-	    HMAC_Update(ctx, phe->ptr, (int)phe->len) == 0 ||
+	    HMAC_Update(ctx, npe->ptr, npe->len) == 0 ||
+	    HMAC_Update(ctx, phe->ptr, phe->len) == 0 ||
 	    HMAC_Final(ctx, dgst, &dgst_len) == 0 || dgst_len != 32) {
 		fido_log_debug("%s: HMAC", __func__);
 		goto fail;
@@ -709,7 +749,6 @@ cbor_encode_change_pin_auth(const fido_blob_t *key, const fido_blob_t *new_pin,
 		goto fail;
 	}
 
-	ok = 0;
 fail:
 	fido_blob_free(&npe);
 	fido_blob_free(&ph);
@@ -720,13 +759,6 @@ fail:
 		HMAC_CTX_free(ctx);
 #endif
 
-	if (ok < 0) {
-		if (item != NULL) {
-			cbor_decref(&item);
-			item = NULL;
-		}
-	}
-
 	return (item);
 }
 
@@ -748,7 +780,7 @@ cbor_encode_set_pin_auth(const fido_blob_t *key, const fido_blob_t *pin)
 	}
 
 	if ((md = EVP_sha256()) == NULL || key->len != 32 || HMAC(md, key->ptr,
-	    (int)key->len, pe->ptr, (int)pe->len, dgst, &dgst_len) == NULL ||
+	    (int)key->len, pe->ptr, pe->len, dgst, &dgst_len) == NULL ||
 	    dgst_len != SHA256_DIGEST_LENGTH) {
 		fido_log_debug("%s: HMAC", __func__);
 		goto fail;
@@ -849,9 +881,7 @@ cbor_encode_hmac_secret_param(const fido_blob_t *ecdh, const es256_pk_t *pk,
 	}
 
 fail:
-	for (size_t i = 0; i < 3; i++)
-		if (argv[i] != NULL)
-			cbor_decref(&argv[i]);
+	cbor_vector_free(argv, nitems(argv));
 
 	if (param != NULL)
 		cbor_decref(&param);
@@ -1082,26 +1112,35 @@ fail:
 static int
 decode_extension(const cbor_item_t *key, const cbor_item_t *val, void *arg)
 {
-	int	*authdata_ext = arg;
-	char	*type = NULL;
-	int	 ok = -1;
+	fido_cred_ext_t	*authdata_ext = arg;
+	char		*type = NULL;
+	int		 ok = -1;
 
-	if (cbor_string_copy(key, &type) < 0 || strcmp(type, "hmac-secret")) {
+	if (cbor_string_copy(key, &type) < 0) {
 		fido_log_debug("%s: cbor type", __func__);
 		ok = 0; /* ignore */
 		goto out;
 	}
 
-	if (cbor_isa_float_ctrl(val) == false ||
-	    cbor_float_get_width(val) != CBOR_FLOAT_0 ||
-	    cbor_is_bool(val) == false || *authdata_ext != 0) {
-		fido_log_debug("%s: cbor type", __func__);
-		goto out;
+	if (strcmp(type, "hmac-secret") == 0) {
+		if (cbor_isa_float_ctrl(val) == false ||
+		    cbor_float_get_width(val) != CBOR_FLOAT_0 ||
+		    cbor_is_bool(val) == false) {
+			fido_log_debug("%s: cbor type", __func__);
+			goto out;
+		}
+		if (cbor_ctrl_value(val) == CBOR_CTRL_TRUE)
+			authdata_ext->mask |= FIDO_EXT_HMAC_SECRET;
+	} else if (strcmp(type, "credProtect") == 0) {
+		if (cbor_isa_uint(val) == false ||
+		    cbor_int_get_width(val) != CBOR_INT_8) {
+			fido_log_debug("%s: cbor type", __func__);
+			goto out;
+		}
+		authdata_ext->mask |= FIDO_EXT_CRED_PROTECT;
+		authdata_ext->prot = cbor_get_uint8(val);
 	}
 
-	if (cbor_ctrl_value(val) == CBOR_CTRL_TRUE)
-		*authdata_ext |= FIDO_EXT_HMAC_SECRET;
-
 	ok = 0;
 out:
 	free(type);
@@ -1110,7 +1149,8 @@ out:
 }
 
 static int
-decode_extensions(const unsigned char **buf, size_t *len, int *authdata_ext)
+decode_extensions(const unsigned char **buf, size_t *len,
+    fido_cred_ext_t *authdata_ext)
 {
 	cbor_item_t		*item = NULL;
 	struct cbor_load_result	 cbor;
@@ -1118,8 +1158,9 @@ decode_extensions(const unsigned char **buf, size_t *len, int *authdata_ext)
 
 	fido_log_debug("%s: buf=%p, len=%zu", __func__, (const void *)*buf,
 	    *len);
+	fido_log_xxd(*buf, *len);
 
-	*authdata_ext = 0;
+	memset(authdata_ext, 0, sizeof(*authdata_ext));
 
 	if ((item = cbor_load(*buf, *len, &cbor)) == NULL) {
 		fido_log_debug("%s: cbor_load", __func__);
@@ -1129,7 +1170,6 @@ decode_extensions(const unsigned char **buf, size_t *len, int *authdata_ext)
 
 	if (cbor_isa_map(item) == false ||
 	    cbor_map_is_definite(item) == false ||
-	    cbor_map_size(item) != 1 ||
 	    cbor_map_iter(item, authdata_ext, decode_extension) < 0) {
 		fido_log_debug("%s: cbor type", __func__);
 		goto fail;
@@ -1204,7 +1244,7 @@ fail:
 int
 cbor_decode_cred_authdata(const cbor_item_t *item, int cose_alg,
     fido_blob_t *authdata_cbor, fido_authdata_t *authdata,
-    fido_attcred_t *attcred, int *authdata_ext)
+    fido_attcred_t *attcred, fido_cred_ext_t *authdata_ext)
 {
 	const unsigned char	*buf = NULL;
 	size_t			 len;
@@ -1227,6 +1267,7 @@ cbor_decode_cred_authdata(const cbor_item_t *item, int cose_alg,
 	len = cbor_bytestring_length(item);
 
 	fido_log_debug("%s: buf=%p, len=%zu", __func__, (const void *)buf, len);
+	fido_log_xxd(buf, len);
 
 	if (fido_buf_read(&buf, &len, authdata, sizeof(*authdata)) < 0) {
 		fido_log_debug("%s: fido_buf_read", __func__);
@@ -1242,7 +1283,7 @@ cbor_decode_cred_authdata(const cbor_item_t *item, int cose_alg,
 	}
 
 	if (authdata_ext != NULL) {
-		if ((authdata->flags & CTAP_AUTHDATA_EXT_DATA) != 0 && 
+		if ((authdata->flags & CTAP_AUTHDATA_EXT_DATA) != 0 &&
 		    decode_extensions(&buf, &len, authdata_ext) < 0)
 			return (-1);
 	}
@@ -1316,6 +1357,7 @@ decode_attstmt_entry(const cbor_item_t *key, const cbor_item_t *val, void *arg)
 {
 	fido_attstmt_t	*attstmt = arg;
 	char		*name = NULL;
+	int		 cose_alg = 0;
 	int		 ok = -1;
 
 	if (cbor_string_copy(key, &name) < 0) {
@@ -1326,11 +1368,16 @@ decode_attstmt_entry(const cbor_item_t *key, const cbor_item_t *val, void *arg)
 
 	if (!strcmp(name, "alg")) {
 		if (cbor_isa_negint(val) == false ||
-		    cbor_int_get_width(val) != CBOR_INT_8 ||
-		    cbor_get_uint8(val) != -COSE_ES256 - 1) {
+		    cbor_get_int(val) > UINT16_MAX) {
 			fido_log_debug("%s: alg", __func__);
 			goto out;
 		}
+		if ((cose_alg = -(int)cbor_get_int(val) - 1) != COSE_ES256 &&
+		    cose_alg != COSE_RS256 && cose_alg != COSE_EDDSA) {
+			fido_log_debug("%s: unsupported cose_alg=%d", __func__,
+			    cose_alg);
+			goto out;
+		}
 	} else if (!strcmp(name, "sig")) {
 		if (cbor_bytestring_copy(val, &attstmt->sig.ptr,
 		    &attstmt->sig.len) < 0) {
