@@ -111,6 +111,18 @@ print_maxmsgsiz(uint64_t maxmsgsiz)
 	printf("maxmsgsiz: %d\n", (int)maxmsgsiz);
 }
 
+static void
+print_maxcredcntlst(uint64_t maxcredcntlst)
+{
+	printf("maxcredcntlst: %d\n", (int)maxcredcntlst);
+}
+
+static void
+print_maxcredidlen(uint64_t maxcredidlen)
+{
+	printf("maxcredlen: %d\n", (int)maxcredidlen);
+}
+
 static void
 print_fwversion(uint64_t fwversion)
 {
@@ -202,6 +214,12 @@ token_info(int argc, char **argv, char *path)
 	/* print maximum message size */
 	print_maxmsgsiz(fido_cbor_info_maxmsgsiz(ci));
 
+	/* print maximum number of credentials allowed in credential lists */
+	print_maxcredcntlst(fido_cbor_info_maxcredcntlst(ci));
+
+	/* print maximum length of a credential ID */
+	print_maxcredidlen(fido_cbor_info_maxcredidlen(ci));
+
 	/* print firmware version */
 	print_fwversion(fido_cbor_info_fwversion(ci));
 
@@ -214,6 +232,11 @@ token_info(int argc, char **argv, char *path)
 	else
 		printf("pin retries: %d\n", retrycnt);
 
+	if (fido_dev_get_uv_retry_count(dev, &retrycnt) != FIDO_OK)
+		printf("uv retries: undefined\n");
+	else
+		printf("uv retries: %d\n", retrycnt);
+
 	bio_info(dev);
 
 	fido_cbor_info_free(&ci);
